/* TODO - Implement the functions word_reverse then words_initialize */



#include <stdlib.h>
#include <ctype.h>


void word_reverse( char* str ){
    /* ROLE         Reverses the order of the character in the string. Does 
                     not modify the end of string marker '\0'. 
        PARAMETERS   str - the string to reverse
    */ 
     
     
    /* temporary implementation does not reverse words, just replace them 
		with "********"
	*/
     
    while(*str){
                *str = '*';
                str++;
    }
}



int is_separator( char data ){
    /* ROLE         Tests if the character passed as parameter is a separator; 
                    i.e. something different than a letter or digit.
       RETURNS      1 - if data is a separator 
                    0 - otherwize
       PARAMETERS   data - the character to test
    */  
    
    return ( !isalnum(data) ); 
     
}

int words_initialize( char* str, char* words[], int maxwords ){
    /* ROLE         Parses the string str and identify each word in it. 
                    Words are separated by any character which makes the 
                    is_separator function returns true. 
                    The address of the first character of each word in 
                    the string str, is stored as a pointer in the array words. 
                    When we find the end of the word we just added in the string
                    str, we modify it so that the next character becomes '\0'. 
                    We are not able to handle more than maxwords words
                    and subsequent ones will be ignored. 
                    Our function will return the number of words it found
                    which also indicates how many of the maxwords elements in 
                    the array words are meaningful. 
                    
       RETURNS      the number of words identified in the string input. 
       PARAMETERS   input    - string in which we will identify words 
                    words    - an array of pointer on strings which will be the 
                               words we have identified  
                    maxwords - the size of the words array of pointers
    */  
    
    
     
	/* this temporary implementation assumes that there is a single word in str
		and that it starts at the first character and ends at the end of str. 
		we don't have to worry about adding a'\0' after the end of this first 
		word since we assume the input string str is perfect for this first 
		implementation. 
	*/
   
	words[0] = &str[0]; 
	return 1; 
}







void word_handle_marker(char* str){ 
    /* 	ROLE         Replaces the end of string marker '\0' by a space ' '
        PARAMETERS   str - the string to modify
    */
     
    while(*str){ str++; }
    *str = ' ';                     
}


void words_modify(char* str){
    /* 	ROLE         Isolates each word in the string parameter and reverse 
                     them individually before to restore the structure of the 
                     original string.
        PARAMETERS   str - the string to modify
    */
     
    /* we only handle this many words at maximum. */
    const int MAXWORDS = 256; 
     
    /* the address of the first character of each word will be stored here */
    char* allwords[MAXWORDS]; 
     
    /* just a loop index variable  */
    int i; 
     
    /* the number of words we identified in str  */
    int nbwords; 
     
    /* to start off with, all identified words are NULL strings */
    for(i=0; i < MAXWORDS ; i++)
			allwords[i] = NULL; 
     
    /* we populate the allwords array 
		str is modified to be segmented in several '\0' terminated substrings
	*/
    nbwords = words_initialize(str, allwords, MAXWORDS); 
     
    /* reverse each substring individually */
    for(i=0 ; i < nbwords ; i++){
            word_reverse(allwords[i]);    
    }
     
    /*	remove the '\0' which broke down str into substrings to restore it 
		as one single string 
	*/
    for(i=0 ; i < (nbwords-1) ; i++){
            word_handle_marker(allwords[i]);        
    }
}

